package net.shrine.adapter.audit

import net.shrine.config.ConfigSource
import net.shrine.util.ShouldMatchersForJUnit
import org.junit.{After, Before, Test}

/**
 * @author david 
 * @since 8/25/15
 */
class AdapterAuditDbTest extends ShouldMatchersForJUnit {// with TestWithDatabase {

  val resultsSent: ResultSent = ResultSent(-1,-1,"ben's query",System.currentTimeMillis())
  val executionStarted: ExecutionStarted = ExecutionStarted(-1,"ben's query",System.currentTimeMillis())
  val executionCompleted: ExecutionCompleted = ExecutionCompleted(-1,-1,"ben's query",System.currentTimeMillis())
  val queryReceived: QueryReceived = QueryReceived("example.com","ben",-1,"ben's query",System.currentTimeMillis(),System.currentTimeMillis(),Some("ben domain"))

  @Test
  def testInsertResultsSent():Unit = {

    ConfigSource.configForBlock("shrine.adapter.audit.useQepAudit","true",this.getClass.getSimpleName){

      AdapterAuditDb.db.insertResultSent(resultsSent)

      val results = AdapterAuditDb.db.selectAllResultsSent
      results should equal(Seq(resultsSent))
    }
  }

  @Test
  def testInsertExecutionStarted():Unit = {

    ConfigSource.configForBlock("shrine.adapter.audit.useQepAudit","true",this.getClass.getSimpleName){

      AdapterAuditDb.db.insertExecutionStarted(executionStarted)

      val results = AdapterAuditDb.db.selectAllExecutionStarts
      results should equal(Seq(executionStarted))
    }
  }

  @Test
  def testInsertExecutionCompleted():Unit = {

    ConfigSource.configForBlock("shrine.adapter.audit.useQepAudit","true",this.getClass.getSimpleName){

      AdapterAuditDb.db.insertExecutionCompleted(executionCompleted)

      val results = AdapterAuditDb.db.selectAllExecutionCompletes
      results should equal(Seq(executionCompleted))
    }
  }

  @Test
  def testInsertQueryReceived():Unit = {

    ConfigSource.configForBlock("shrine.adapter.audit.useQepAudit","true",this.getClass.getSimpleName){

      AdapterAuditDb.db.insertQueryReceivedIO(queryReceived).unsafeRunSync()

      val results = AdapterAuditDb.db.selectAllQueriesReceived
      results should equal(Seq(queryReceived))
    }
  }

  @Before
  def beforeEach(): Unit = {
    AdapterAuditDb.db.createTables()
  }

  @After
  def afterEach(): Unit = {
    AdapterAuditDb.db.dropTables()
  }

}
