package net.shrine.aim3

import cats.effect.IO
import javax.xml.datatype.XMLGregorianCalendar
import net.shrine.adapter.dao.{AdapterQueryHistoryDb, Category, SortOrder, Strategy, UserNameQueryOptions}
import net.shrine.adapter.dao.model.ShrineQuery
import net.shrine.audit.UserName

/**
 * @author clint
 * @since Apr 24, 2013
 */
final class I2b2AdminDao(shrineProjectId: String) {


  def findQueriesByRequestIO(request:ReadI2b2AdminPreviousQueriesRequest): IO[Seq[ShrineQuery]] = {
    findQueriesByUserFlaggedStatusDateAndSearchStringIO(
      request.username,
      request.categoryToSearchWithin,
      request.searchString,
      request.maxResults,
      request.startDate,
      request.searchStrategy,
      request.sortOrder
    )
  }

  def findQueriesByUserFlaggedStatusDateAndSearchStringIO(
                                                           username: UserNameQueryOptions,
                                                           category: Category,
                                                           searchString: String,
                                                           howMany: Int,
                                                           startDate: Option[XMLGregorianCalendar],
                                                           strategy: Strategy,
                                                           sortOrder: SortOrder
                                                       ): IO[Seq[ShrineQuery]] = {
    AdapterQueryHistoryDb.db.previousQueriesByUserFlaggedStatusAndSearchStringIO(username, category, searchString, startDate, sortOrder, strategy).map(_.take(howMany).toSeq)
  }
  
  def findQueryingUsersByProjectIdIO(projectId: String): IO[Seq[I2b2AdminUserWithRole]] = {

    def toI2b2AdminUserWithRole(username: UserName) = I2b2AdminUserWithRole(shrineProjectId, username, "USER")

    AdapterQueryHistoryDb.db.getUsersByProjectIdIO(projectId).map(p => p.map(toI2b2AdminUserWithRole).toIndexedSeq)
  }
}