package net.shrine.aim3

import net.shrine.protocol.i2b2.serialization.I2b2UnmarshallingHelpers
import net.shrine.protocol.i2b2.{AuthenticationInfo, RequestType, ResultOutputType, ShrineRequest}
import net.shrine.xml.{NodeSeqEnrichments, XmlUtil}
import net.shrine.adapter.i2b2Protocol.ShrineRequest

import scala.concurrent.duration.Duration
import scala.util.Try
import scala.xml.NodeSeq

/**
 * @author clint
 * @since Jan 10, 2014
 */
final case class ReadI2b2AdminQueryingUsersRequest(
                                                    override val projectId: String,
                                                    override val waitTime: Duration,
                                                    override val authn: AuthenticationInfo,
                                                    projectIdToQueryFor: String
                                                  ) extends ShrineRequest(projectId, waitTime, authn)  {

  override val requestType: RequestType = RequestType("ReadI2b2AdminPreviousQueriesRequest")
  
  protected override def i2b2MessageBody: NodeSeq = XmlUtil.stripWhitespace {
    <message_body>
      <ns99:get_all_role>
        <project_id>{ projectIdToQueryFor }</project_id>
      </ns99:get_all_role>
    </message_body>
  }

  override def toXml: NodeSeq = XmlUtil.stripWhitespace {
    <readAdminQueryingUsers>
	  { headerFragment }
      <projectId>{ projectIdToQueryFor }</projectId>
    </readAdminQueryingUsers>
  }
}

object ReadI2b2AdminQueryingUsersRequest extends I2b2UnmarshallingHelpers {

  def isI2b2AdminQueryingUsersRequest(requestXml: NodeSeq): Boolean = ShrineRequest.hasMessageBodySubElement(requestXml, "get_all_role")

  def fromI2b2(breakdownTypes: Set[ResultOutputType])(xml: NodeSeq): Try[ReadI2b2AdminQueryingUsersRequest] = {
    import NodeSeqEnrichments.Strictness._

    for {
      projectId <- i2b2ProjectId(xml)
      waitTime <- i2b2WaitTime(xml)
      authn <- i2b2AuthenticationInfo(xml)
      projectIdToQueryFor <- (xml withChild "message_body" withChild "get_all_role" withChild "project_id").map(_.text.trim)
    } yield {
      net.shrine.aim3.ReadI2b2AdminQueryingUsersRequest(projectId, waitTime, authn, projectIdToQueryFor)
    }
  }
}