package net.shrine.aim3

import net.shrine.adapter.i2b2Protocol.ShrineResponse
import net.shrine.protocol.i2b2.serialization.I2b2Unmarshaller
import net.shrine.xml.XmlUtil

import scala.xml.NodeSeq

/**
 * @author clint
 * @since Jan 10, 2014
 */
final case class ReadI2b2AdminQueryingUsersResponse(users: Seq[I2b2AdminUserWithRole]) extends ShrineResponse {
  
  //For tests
  private[aim3] def i2b2Body: NodeSeq = i2b2MessageBody
  
  override protected def i2b2MessageBody: NodeSeq = XmlUtil.stripWhitespace {
    <ns6:roles>
	  { users.map(_.toI2b2) }
	</ns6:roles>
  }
  
  override def toXml: NodeSeq = XmlUtil.stripWhitespace {
    <readI2b2AdminQueryingUsersResponse>
      <users>
	    { users.map(_.toXml) }
	  </users>
	</readI2b2AdminQueryingUsersResponse>
  }
}

object ReadI2b2AdminQueryingUsersResponse extends I2b2Unmarshaller[ReadI2b2AdminQueryingUsersResponse] {
  //NB: Only needed for tests :/
  override def fromI2b2(xml: NodeSeq): ReadI2b2AdminQueryingUsersResponse = {
    //NB: Fail fast
    require((xml \ "response_header" \ "result_status" \ "status" \ "@type").text == "DONE")
    
    val usersWithRoles = (xml \ "message_body" \ "roles" \ "role").map { userWithRoleXml =>
      val projectId = (userWithRoleXml \ "project_id").text.trim
      val username = (userWithRoleXml \ "user_name").text.trim
      val role = (userWithRoleXml \ "role").text.trim

      I2b2AdminUserWithRole(projectId, username, role)
    }

    ReadI2b2AdminQueryingUsersResponse(usersWithRoles)
  }
}