package net.shrine.http4s.servlet

import cats.data.Kleisli
import cats.effect.{ConcurrentEffect, ContextShift}
import net.shrine.log.Log
import org.http4s.{HttpRoutes, Request, Response}

import scala.language.higherKinds

object Http4sRequestResponseLogger {

  def apply[F[_]: ConcurrentEffect: ContextShift](service: HttpRoutes[F]): HttpRoutes[F] = Kleisli { request: Request[F] =>
    logRequest(request)

    service(request).map {response: Response[F] =>
      logRequestResponse(request,response)
      response
    }
  }

  def logRequest[F[_]: ConcurrentEffect: ContextShift](request:Request[F]): Request[F] = {
    Log.info(s"Request received: $request")
    request
  }

  def logRequestResponse[F[_]: ConcurrentEffect: ContextShift](request:Request[F],response: Response[F]): Response[F] = {
    Log.info(
      s"""Request: $request
         |Response: $response
       """.stripMargin)
    response
  }
}
