package net.shrine.http4s.servlet

import scala.annotation.nowarn
import cats.effect.{Blocker, ConcurrentEffect, ContextShift}

import javax.servlet.{ServletContext, ServletRegistration}
import org.http4s.server.DefaultServiceErrorHandler
import net.shrine.config.{ConfigExtensions, ConfigSource}
import net.shrine.http4s.catsio.ExecutionContexts
import org.http4s.headers.`Strict-Transport-Security`
import org.http4s.server.middleware.HSTS
import org.http4s.servlet.{AsyncHttp4sServlet, BlockingServletIo}
import org.http4s.{HttpApp, HttpRoutes}

import scala.concurrent.duration.{Duration, DurationInt}

/**
 * This code began as a copy of http4s' ServletContextSyntax. It adds request/response logging and a default
 * timeout from shrine.
 */
object ShrineServletContextSyntax {

   private[servlet] def wrapResponseHeaders[F[_]: ConcurrentEffect: ContextShift](service: HttpRoutes[F]) = {
     val serviceWithLogging = Http4sRequestResponseLogger(service)

     val hstsHeader = `Strict-Transport-Security`.unsafeFromDuration(
       maxAge = 365.days, //if preload is true, then 2 years is the recommended time according to https://hstspreload.org
       includeSubDomains = true,
       preload = false //if set to true, need to submit site to https://hstspreload.org for preload to work
     )
     HSTS(serviceWithLogging, hstsHeader)
   }
  /** Wraps an [[HttpRoutes]] and mounts it as an [[AsyncHttp4sServlet]]
   *
   * Assumes non-blocking servlet IO is available, and thus requires at least Servlet 3.1.
   */
  def mountService[F[_]: ConcurrentEffect: ContextShift](
                                                          context: ServletContext,
                                                          name: String,
                                                          service: HttpRoutes[F],
                                                          mapping: String = "/*"): ServletRegistration.Dynamic = {
    val wrappedService = wrapResponseHeaders(service)
    import org.http4s.syntax.all._
    mountHttpApp(context,name, wrappedService.orNotFound, mapping)
  }

  @nowarn("cat=unused")
  def mountHttpApp[F[_]: ConcurrentEffect: ContextShift](
                                                          context: ServletContext,
                                                          name: String,
                                                          service: HttpApp[F],
                                                          mapping: String = "/*"): ServletRegistration.Dynamic = {

    //TODO: find out why AutoSlash middleware does not work - SHRINE-2581
    val servlet = new AsyncHttp4sServlet(
      service = service,
      asyncTimeout = ConfigSource.config.get("shrine.api.asyncTimeout",Duration(_)),

//      servletIo = NonBlockingServletIo(4096), todo switch back to NonBlocking when https://github.com/http4s/http4s/issues/2354 is fixed
      servletIo = BlockingServletIo(4096,Blocker.liftExecutionContext(ExecutionContexts.dispatchExecutionContext)),
      serviceErrorHandler = DefaultServiceErrorHandler[F]
    )
    val reg = context.addServlet(name, servlet)
    reg.setLoadOnStartup(1)
    reg.setAsyncSupported(true)
    reg.addMapping(mapping)
    reg
  }
}