#!/usr/bin/env bash

echo "EXTERNAL_PORT: $EXTERNAL_PORT"
echo "ZOOKEEPER_HOSTS: $ZOOKEEPER_HOSTS"
echo "KAFKA_HOSTNAME: $KAFKA_HOSTNAME"
echo "BROKER_ID: $BROKER_ID"

# Split the ZOOKEEPER_HOSTS comma-separated list into an array
IFS=','
read -a ZOOKEEPER_HOSTS_ARR <<< "$ZOOKEEPER_HOSTS"

#Add user admin and shrine node users to zookeeper
bin/kafka-configs.sh --zookeeper ${ZOOKEEPER_HOSTS_ARR[0]} --alter --add-config 'SCRAM-SHA-256=[iterations=8192,password=shrineHub-secret],SCRAM-SHA-512=[password=shrineHub-secret]' --entity-type users --entity-name shrineHubUser
bin/kafka-configs.sh --zookeeper ${ZOOKEEPER_HOSTS_ARR[0]} --alter --add-config 'SCRAM-SHA-256=[iterations=8192,password=shrineNode1-secret],SCRAM-SHA-512=[password=shrineNode1-secret]' --entity-type users --entity-name shrineNode1User
bin/kafka-configs.sh --zookeeper ${ZOOKEEPER_HOSTS_ARR[0]} --alter --add-config 'SCRAM-SHA-256=[iterations=8192,password=shrineNode2-secret],SCRAM-SHA-512=[password=shrineNode2-secret]' --entity-type users --entity-name shrineNode2User
bin/kafka-configs.sh --zookeeper ${ZOOKEEPER_HOSTS_ARR[0]} --alter --add-config 'SCRAM-SHA-256=[password=admin-secret],SCRAM-SHA-512=[password=admin-secret]' --entity-type users --entity-name admin


#Start Kafka
export KAFKA_OPTS="-Djava.security.auth.login.config=/tmp/kafka_server_jaas.conf"
nohup bin/kafka-server-start.sh config/server.properties --override log.dirs=/tmp/kafka-logs1 \
--override broker.id=$BROKER_ID \
--override zookeeper.connect="${ZOOKEEPER_HOSTS}" \
--override listeners="CLIENT://:19092,EXTERNAL://:$EXTERNAL_PORT" \
--override advertised.listeners="CLIENT://$KAFKA_HOSTNAME:19092,EXTERNAL://localhost:$EXTERNAL_PORT" \
--override listener.name.client.scram-sha-512.sasl.jaas.config="org.apache.kafka.common.security.scram.ScramLoginModule required \
    username='admin' password='admin-secret' \
    user_admin='admin-secret' \
    user_shrine='shrine-secret';" \
 --override ssl.keystore.location=/usr/local/kafka/truststore/$KAFKA_HOSTNAME/shrine.keystore \
 --override ssl.truststore.location=/usr/local/kafka/truststore/$KAFKA_HOSTNAME/shrine.keystore \
 --override listener.name.external.scram-sha-512.sasl.jaas.config="org.apache.kafka.common.security.scram.ScramLoginModule required \
    username='admin' \
    password='admin-secret' \
    user_admin='admin-secret' \
    user_shrine='shrine-secret';" &

/tmp/wait-for-it.sh ${KAFKA_HOSTNAME}:19092 -t 0
sleep 80s

tail -f /dev/null
