package net.shrine.hub.metrics

import com.opencsv.CSVWriter
import net.shrine.hub.data.store.HubDb
import net.shrine.protocol.version.{DateStamp, NodeId}
import net.shrine.protocol.version.v2.{Node, NodeSystemSpec}

import java.io.File

object NodeCsv {

  def writeNodeCsv(file:File,startDate:DateStamp,endDate:DateStamp):Unit = CommonMaps.writeCsv (file,{ csvWriter: CSVWriter =>
    csvWriter.writeNext(Column.header(NodeRow.columns))

    val nodes: Seq[Node] = HubDb.db.selectLatestNodesIO.unsafeRunSync().map(_.get).toSeq
    val nodeIdToNodeSystemSpecs: Map[NodeId, NodeSystemSpec] = HubDb.db.selectLatestNodeSystemSpecsIO.unsafeRunSync().
      map(_.get).map(n => n.id -> n ).toMap

    nodes.foreach{node =>
      val nodeRow = NodeRow(node,nodeIdToNodeSystemSpecs.get(node.id))
      csvWriter.writeNext(Column.rowForColumns(NodeRow.columns)(nodeRow))
    }
  })
}

case class NodeRow (  
                      node:Node,
                      nodeSystemSpec:Option[NodeSystemSpec],
                    )

object NodeRow{

  val columns: Array[Column[NodeRow]] = Array(
    Column("id", { nr: NodeRow => nr.node.id.excelSafeString }),
    Column("name", { nr: NodeRow => nr.node.name.underlying}),
    Column("key", { nr: NodeRow => nr.node.key.underlying }),
    Column("user domain", { nr: NodeRow => nr.node.userDomainName.underlying }),
    Column("mom queue", { nr: NodeRow => nr.node.momQueueName.underlying }),
    Column("mom id", { nr: NodeRow => nr.node.momId }),
    Column("admin email", { nr: NodeRow => nr.node.adminEmail }),
    Column("send queries", { nr: NodeRow => nr.node.sendQueries.toString }),
    Column("understands protocol", { nr: NodeRow => nr.node.understandsProtocol.underlying.toString }),
    Column("created on", { nr: NodeRow => nr.node.versionInfo.createDate.toDateString }),
    Column("last changed on", { nr: NodeRow => nr.node.versionInfo.changeDate.toDateString }),
    Column("version", { nr: NodeRow => nr.node.versionInfo.itemVersion.underlying.toString }),
    Column("Node spec last changed", { nr: NodeRow => nr.nodeSystemSpec.map(_.versionInfo.changeDate.toDateString).getOrElse("") }),
    Column("Node spec version", { nr: NodeRow => nr.nodeSystemSpec.map(_.versionInfo.itemVersion.underlying.toString).getOrElse("") }),
    Column("SHRINE Version", { nr: NodeRow => nr.nodeSystemSpec.map(_.versionInfo.shrineVersion.underlying).getOrElse("") }),
    Column("SHRINE Tomcat JDK Version", { nr: NodeRow => nr.nodeSystemSpec.map(_.shrineJdk).getOrElse("") }),
    Column("SHRINE Tomcat OS", { nr: NodeRow => nr.nodeSystemSpec.map(_.shrineOperatingSystem).getOrElse("") }),
    Column("SHRINE Database Brand", { nr: NodeRow => nr.nodeSystemSpec.map(_.shrineDatabaseBrand).getOrElse("") }),
  )
}
