import React, { useState, useEffect } from "react";
import PropTypes from "prop-types";

import { Dialog, Button, Typography } from "@material-ui/core";

import { ValidatedTextField } from "components";
import "./QueryNameDialog.scss";

export const QueryNameDialog = ({ open, onClose, currentQueryName }) => {
  const [canSubmit, setCanSubmit] = useState(false);
  const [queryName, setQueryName] = useState("");

  const resetQueryName = () => {
    if (currentQueryName) {
      setQueryName(currentQueryName);
    } else {
      setQueryName("");
    }
  };

  const handleTextChange = (textValue, textValid) => {
    setQueryName(textValue);
    setCanSubmit(textValue !== currentQueryName && textValid);
  };

  const handleCancel = () => {
    resetQueryName();
    onClose(false);
  };

  const handleRenameQuery = () => {
    if (canSubmit) {
      onClose(true, queryName);
    }
  };

  useEffect(() => {
    resetQueryName();
  }, [currentQueryName]);

  useEffect(() => {
    setQueryName(currentQueryName);
  }, [currentQueryName]);

  return (
    <Dialog className="QueryNameDialog" open={open} onClose={handleCancel}>
      <div className="DialogContent">
        <span className="close-button" onClick={handleCancel} role="button">
          <i className="fa fa-times" aria-hidden="true" />
        </span>
        <div className="title-label-wrapper rename-criteria">
          <Typography className="title">Rename Criteria Set</Typography>
        </div>
        <div>
          <ValidatedTextField
            // Note: \n characters are disallowed by virtue of the "multiline" property being false
            multiline={false}
            autoFocus
            label="Name"
            fullWidth
            value={queryName}
            onTextChange={handleTextChange}
            className="query-name-input" // needed by unit spec
          />
        </div>
        <div className="button-row">
          <Button
            onClick={handleRenameQuery}
            className="btn-primary submit-button"
            disabled={!canSubmit}
          >
            Update
          </Button>
        </div>
      </div>
    </Dialog>
  );
};

QueryNameDialog.propTypes = {
  open: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  currentQueryName: PropTypes.string.isRequired
};
