import React from "react";
import { mount } from "enzyme";
import { shallowToJson } from "enzyme-to-json";
import { QueryNameDialog } from "../QueryNameDialog";

describe("QueryNameDialog", () => {
  const props = {
    open: true,
    currentQueryName: "my query name",
    onClose: jest.fn()
  };

  it("should not call onClose when submit button is clicked but the query name has not been changed", () => {
    const output = mount(<QueryNameDialog {...props} />);
    output
      .find(".submit-button")
      .at(1)
      .simulate("click");

    expect(props.onClose).not.toHaveBeenCalled();
  });

  it("should call onClose when submit button is clicked and the query name has been changed", () => {
    const output = mount(<QueryNameDialog {...props} />);
    output
      .find("div.query-name-input input[type='text']")
      .at(0)
      .simulate("change", { target: { value: "my query name modified" } });
    output
      .find(".submit-button")
      .at(0)
      .simulate("click");

    expect(props.onClose).toHaveBeenCalled();
  });

  xit("should render correctly", () => {
    const output = mount(<QueryNameDialog {...props} />);
    expect(shallowToJson(output)).toMatchSnapshot();
  });
});
