import { call, takeLatest, put } from "redux-saga/effects";
import { FLAG_QUERY, unexpectedError } from "../actions";
import {getBaseUrl, secureFetchOrLogout} from "../utilities";

export function* doFlagQuery(action) {
  const { queryId, flagMessage, flagged } = action.payload;
  const flagBody = flagged ? { message: flagMessage, flagged } : { flagged };
  const url = `${getBaseUrl()}qep/changeQueryFlag/${queryId}`;
  const fetchConfig = {
    headers: { "Content-Type": "application/json" },
    method: "POST",
    body: JSON.stringify(flagBody)
  };

  try {
    const response = yield call(secureFetchOrLogout, url, fetchConfig);
    if (!response.ok) {
      yield put(unexpectedError(response));
    }
  } finally {
    const msg = `fetch of ${url} thread closed`;
    yield msg;
  }
}

export function* flagQuerySaga() {
  yield takeLatest(FLAG_QUERY, doFlagQuery);
}
